require "./selenium_spec/spec_helper.rb"

ALL_TAB_NAMES = ['Summary', 'package', 'puppetral', 'rpcutil', 'service']
TASK_TABS = ALL_TAB_NAMES.reject {|x| x == "Summary"}
RUN_ACTION_TIMEOUT = 60
INITIAL_LOAD_TIMEOUT = 20

describe 'the Live Management web UI Advanced Tasks tab' do

  before :all do
    @driver = get_driver
    link = @driver.find_element(:link => "Advanced Tasks")
    @driver.action.click(link).perform
    @menu = @driver.find_element(:css => "div#main div#content ul.navigation")
    @panel = @driver.find_element(:css => "div#content div.panel")
  end

  after :all do
    @driver.quit
  end

  describe 'task type menu' do

    it 'should exist' do
      @menu.displayed?.should == true
    end

    it "should populate within #{INITIAL_LOAD_TIMEOUT} seconds" do
      wait = Selenium::WebDriver::Wait.new(:timeout => INITIAL_LOAD_TIMEOUT, 
                                           :message => "Menu didn't load within #{INITIAL_LOAD_TIMEOUT} seconds")
      wait.until { @menu.find_elements(:tag_name => "li").length > 1 }
      all_tabs = @menu.find_elements(:tag_name => "li")
      @@tabs = all_tabs.select {|tab| tab.displayed?}

    end

    it "should have exactly #{ALL_TAB_NAMES.length} tabs" do
      @@tabs.length.should == ALL_TAB_NAMES.length
    end

    ALL_TAB_NAMES.each_with_index do |tab_name, i|
      it "should have a #{tab_name} tab" do
        @@tabs[i].text.should == tab_name
      end
    end

    it 'should have the Summary tab active by default' do
      active_tab.text.should == "Summary"
    end

    it 'should activate a tab when the tab is clicked' do
      @@tabs.each do |tab|
        link = tab.find_element(:css => "a")
        @driver.action.click(link).perform
        tab.should == active_tab
      end
    end

  end

  describe 'Summary panel' do

    before :all do
      @summary_link = @driver.find_element(:css => "div#main div#content ul.navigation li a")
      @driver.action.click(@summary_link).perform
      @summaries = @panel.find_elements(:class => "summary")
      @summaries = @summaries.select(&:displayed?)
    end

    it 'should have summaries of all task tabs in the same order as the sidebar menu' do
      @summaries.each_with_index do |summary, i|
        task_name = summary.find_element(:css => "a strong").text
        task_name.should == ALL_TAB_NAMES[i + 1]
      end
      @summaries.length.should == ALL_TAB_NAMES.length - 1
    end

    describe 'task summaries' do
      TASK_TABS.each do |tab_name|
        it "should link to the #{tab_name} resource panel" do
          @driver.action.click(@summary_link).perform
          task_link = @panel.find_element(:partial_link_text => tab_name)
          @driver.action.click(task_link).perform
          active_tab.text.should == tab_name
        end
      end
    end

  end

  describe 'task type' do

    TASK_TABS.each_with_index do |tab_name, i|

      describe "#{tab_name} panel" do

        it "should still be in the same position in the tab menu" do
          tab = @@tabs[i + 1]
          tab.text.should == tab_name
          link = tab.find_element(:css => "a")
          @driver.action.click(link).perform
        end

        it "should display #{tab_name} in the title" do
          task_type = @panel.find_element(:css => "div.agent div h2 + h2 strong").text
          task_type.should == tab_name
        end

      end
    end
  end

  def active_tab
    active_tabs = @menu.find_elements(:class => "active")
    raise "There is more than one active tab." if active_tabs.length != 1
    active_tabs.first
  end

end
